/******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************
 *
 * Summary:
 *   A view layout subclass of ContentPane with layout as the following:
 * 
 *     +--------+------------------+---------------------------+--------------------+
 *     |  icon  | left header node |         right header node | right actions area |
 *     |  node  +------------------+---------------------------+--------------------+
 *     |        | type node / content selector                                      |
 *     +--------+--------+----------------------------------------------------------+
 *     | attributes node | status node with icon                                    |
 *     +-----------------+----------------------------------------------------------+
 *     | summary node                                                               |
 *     +----------------------------------------------------------------------------+
 *     | left actions area                                                          |
 *     +----------------------------------------------------------------------------+
 *     |                                                                            |
 *     | main / alt content container node                                          |
 *     |                                                                            |
 *     +----------------------------------------------------------------------------+
 *
 *   The left header are used for in-progress indicator and the title.  The title
 *   can be editable if the attribute "titleEditable" is set to true.  CS Help icon
 *   is located to the right of the title and only visible if the attribute
 *   "contextHelpId" is set.
 *   
 *   The icon take CSS class with a 48x48 pixels image.  It is only visible if the
 *   attribute "iconClass" is set.
 *   
 *   The type node can be used for content selector when dual contents is used. Set
 *   the attribute "useDualContents" to true to have a main or alternate content
 *   container.  Labels must be specified for the content selector for selected and
 *   deselected content container.
 *   
 *   Attributes node area is an opened DIV for consumer to populate.  There are CSS
 *   classes defined for comsumer to use to layout the nodes properly.  The classes
 *   are "attributes-entry", "attributes-entry-label", "attributes-entry-value" for
 *   text value, and "attributes-entry-icon" for icon value.
 *   
 *   Status node with icon is used for server call return status.  There are API's
 *   to set the status message and icon.  Only the status label of "Success...",
 *   "Error...", or "Info..." is shown and the detail message is displayed as a
 *   tooltip when hover over.  The node is hidden when the status is cleared.
 *   
 *   Summary node is another opened DIV for consumer to set the innerHTML for simple
 *   message or append children nodes.
 *   
 *   The left actions area uses action links with icons.  Use the API addLeftViewAction() to
 *   create the action in the area.  The container area is hidden until action(s) is added.
 *   
 *   The right actions area at the top right corner uses plain HTML button.  use the
 *   API addRightViewAction to create action in the area.
 *   
 */

dojo.provide("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.InlineEditBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.Textarea");
dojo.require("jazz.ui.HoverPopup");
dojo.require("dojox.xml.parser");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.util.ResizeNotifier");

dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.InlineEditor");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormFieldGroup");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.BidiUtil");

(function() {
	
var wordWrapping = com.ibm.asq.common.web.ui.internal.util.wordWrapping;
var _dojo = dojo;
var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
var TRIM_MAX = 100;

var BidiUtil = com.ibm.asq.common.web.ui.internal.util.BidiUtil;

// Summary: 
//    A simple text area that acts like an inline edit area. 
// 

dojo.declare("HeaderLayout.PairButtons", null, {
    headerButton: null,
    bottomButton: null,
    isButtonWidget: false,
	id: null,
	
    constructor: function(headerButton, bottomButton, isButtonWidget){
        this.headerButton = headerButton;
        this.bottomButton = bottomButton;
		this.isButtonWidget = isButtonWidget;
    },
    
	addClass: function(className) {
		if(this.isButtonWidget) {
			dojo.addClass(this.headerButton.domNode, className);
			dojo.addClass(this.bottomButton.domNode, className);
		} else {
			dojo.addClass(this.headerButton, className);
			dojo.addClass(this.bottomButton, className);
		}
	},
	
	removeClass: function(className) {
		if(this.isButtonWidget) {
			dojo.removeClass(this.headerButton.domNode, className);
			dojo.removeClass(this.bottomButton.domNode, className);
		} else {
			dojo.removeClass(this.headerButton, className);
			dojo.removeClass(this.bottomButton, className);
		}
	},
	
	setClass: function(className) {
		if(this.isButtonWidget) {
			this.headerButton.domNode.className = className;
			this.bottomButton.domNode.className = className;
		} else {
			this.headerButton.className = className;
			this.bottomButton.className = className;
		}
	},
	
	hasClass: function(className) {
		if(this.isButtonWidget) {
			return dojo.hasClass(this.headerButton.domNode, className);
		} else {
			return dojo.hasClass(this.headerButton, className);
		}
	},
	
	addIconClass: function(className) {
		if(this.isButtonWidget) {
			dojo.addClass(this.headerButton.iconNode, className);
			dojo.addClass(this.bottomButton.iconNode, className);
		} 
	},
	
	removeIconClass: function(className) {
		if(this.isButtonWidget) {
			dojo.removeClass(this.headerButton.iconNode, className);
			dojo.removeClass(this.bottomButton.iconNode, className);
		} 
	},
	
	setDisabled: function(disable) {		
		if(this.isButtonWidget) {
			this.headerButton.set('disabled', disable);
			this.bottomButton.set('disabled', disable);
		} else {
			this.headerButton.disabled = disable;
			this.bottomButton.disabled = disable;
		}
        if ((this.headerButton.baseClass == "dijitDropDownButton") && this.headerButton.domNode && this.bottomButton.domNode) {
            dojo.toggleClass(this.headerButton.domNode, "view-action-dropdown-disabled", disable);
            dojo.toggleClass(this.bottomButton.domNode, "view-action-dropdown-disabled", disable);
            this.headerButton.disabled = disable;
            this.bottomButton.disabled = disable;
        }
	},
	
	isDisabled: function() {
		if(this.isButtonWidget) {
			return this.headerButton.isDisabled();
		} else {
			return this.headerButton.disabled;
		}
	},
	
	setId: function(id) {
		this.id = id;
	},
	
	getId: function() {
		return this.id;
	},
	
	setStyle: function(styleName, styleValue) {
		if(this.isButtonWidget) {
			dojo.style(this.headerButton.domNode, styleName, styleValue);
			dojo.style(this.bottomButton.domNode, styleName, styleValue);
		} else {
			dojo.style(this.headerButton, styleName, styleValue);
			dojo.style(this.bottomButton, styleName, styleValue);
		}
	},
	
	setLabel: function(label) {
		if(this.isButtonWidget) {
			this.headerButton.set('label',label);
			this.bottomButton.set('label',label);
		}else{
			this.headerButton.title = label;
			this.bottomButton.title = label;
		}
	}
});

// A simple <a> link widget that behaves as an in-page tab button.
//
// label - display label
// selected - state of selected or not
// group - group this tab button belongs to, only one is selected at a time in the group
// onSelect - method to invoke when selected
//
dojo.declare("ViewHeaderLayout.InPageTabButton", [dijit._Widget, dijit._Templated], {
                    
    templateString: '<div waiRole="tab" class="in-page-tab" tabIndex="-1" dojoAttachEvent="onkeypress:onKeySelect">' + // focus by screen reader only
                       '<a href="javascript:{}" dojoAttachPoint="focusNode" dojoAttachEvent="onclick:onClickSelect,onkeypress:onKeySelect">' +
                          '<div dojoAttachPoint="tabButtontopleft" class="tabButton-topleft-selected">' +
                             '<div dojoAttachPoint="tabButtontopright" class="tabButton-topright-selected">' +
                                '<div dojoAttachPoint="tabButtontop" class="tabButton-top-selected"></div>' +
                             '</div>' +
                          '</div>' +
                          '<div dojoAttachPoint="tabButtonleft" class="tabButton-left-selected">' +
                             '<div dojoAttachPoint="tabButtonright" class="tabButton-right-selected">' +
                                '<label class="content" dojoAttachPoint="labelNode">${label}</label>' +
                             '</div>' +
                          '</div>' +
                       '</a>' +
                    '</div>',
    label:"",
    selected: false,
    
    postCreate: function() {
        this.inherited(arguments);
        if (this.inGroup) {
            this.inGroup.push(this);
        }
        this._updateTabCss();
        this.labelNode.setAttribute("for", this.domNode.id);
    },
    
    _setLabelAttr: function(label) {
    	label = Utility.encodeHTMLTags(label);
        this.labelNode.innerHTML = label;
    },
    
    _setGroupAttr: function(group) {
        this.inGroup = group;
        this.inGroup.push(this);
    },

    _setSelectedAttr: function(selected) {
        this.selected = selected;
        this._updateTabCss();
    },
    
    _setHideAttr: function(hide) {
        dojo.style(this.domNode, "display", hide ? "none" : "inline");
    },
    
    onSelect: function() {
    },
    
    onClickSelect: function(evt) {
        if (!this.selected) {
            // do nothing if it is already selected
            if (this.inGroup) {
                // if in a group only one can be selected
                var selected = this;
                dojo.forEach(this.inGroup, function(tab){
                    tab.selected = (tab == selected);
                    tab._updateTabCss();
                });
            } else {
                this.selected = true;
                this._updateTabCss();
            }
            this.onSelect();
        }
        if (evt){
            dojo.stopEvent(evt);
        }
    },
    
    onKeySelect: function(evt) {
        if (evt && (evt.keyCode == dojo.keys.ENTER || evt.charCode == dojo.keys.SPACE)) {
            this.onClickSelect();
            dojo.stopEvent(evt);
        }
    },

    _updateTabCss: function(){
        // keep all spacing/margin style, just without any background if not selected
        var showDeslected = !this.selected;
        
        dojo.toggleClass(this.focusNode, "tab-selected", this.selected);
        dojo.toggleClass(this.tabButtontopleft, "tab-deselected", showDeslected);
        dojo.toggleClass(this.tabButtontopright, "tab-deselected", showDeslected);
        dojo.toggleClass(this.tabButtontop, "tab-deselected", showDeslected);
        dojo.toggleClass(this.tabButtonleft, "tab-deselected", showDeslected);
        dojo.toggleClass(this.tabButtonright, "tab-deselected", showDeslected);
        
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout",
[dijit.layout.ContentPane, dijit._Templated],
{
    hTabTemplateString: ['<div id="${id}" style="padding: 0px 4px" dojoAttachPoint="domNode" tabindex="-1" class="com-ibm-asq-common-web-ui-view-header">' ,
                    '  <div dojoAttachPoint="viewHeaderBackNode" class="view-header-back" tabindex="-1">' ,
                    '    <div class="view-header-container" tabindex="-1">' ,
                    '      <div dojoAttachPoint="breadcrumbContainerNode">' ,
                    '        <span dojoAttachPoint="breadcrumbNode" dojoType="jazz.ui.Breadcrumbs"></span>' ,
                    '      </div>' ,

                    '      <div class="view-status-outter-container" dojoAttachPoint="wholestatusContainer"> ' ,
                    '        <span dojoAttachPoint="statusContainer" role="status">' ,
                    '          <span dojoType="jazz.ui.MessageBox" dojoAttachPoint="statusLineNode" dojoAttachEvent="onClose:_onMessageClose" tabIndex="-1" aria-live="assertive"></span>' ,
                    '        </span>' ,
                    '      </div> ' ,

                    '      <div dojoAttachPoint="artifactContainer" class="artifact-container">' ,
                    '        <div dojoAttachPoint="iconNode" class="${iconClass}" style="float:left"></div>' ,
                    '        <div tabindex="-1" class="artifact-node">' ,
                    '          <div dojoAttachPoint="titleActionsContainer">' ,
                    '            <div dojoAttachPoint="actionsAreaRight" class="actions-area-right"></div>' ,
                    '            <div dojoAttachPoint="actionsAreaLeft" class="actions-area-left"></div>' ,
                    '            <div dojoAttachPoint="viewHeaderRight" waiRole="region" class="view-right-header" style="float:right"></div>' ,
                    '            <div dojoAttachPoint="progressIndicator" class="progress-indicator" style="display: none">' ,
                    '              <div class="view-progress-image">' ,
                    '                <span dojoAttachPoint="progressLabel" class="view-progress-label"></span>' ,
                    '              </div>' ,
                    '            </div>' ,
                    '            <div dojoAttachPoint="viewHeaderLeft"  class="view-left-header">' ,
          			'			  <span style="display:none" id="${id}_titleAriaLabel">${titleAriaLabel}</span>'+
                    '              <span class="required-indicator" dojoAttachPoint="fieldRequiredNode" style="float:left;margin:0px 2px;display:none" tabindex="-1">&#042;</span>' +
                    '              <label dojoAttachPoint="viewID" style="display:none;margin-right:2px;padding-top:4px;" class="view-title"></label>' ,
                    '  			   <label dojoAttachPoint="viewNonEditableTitle" class="view-title" tabIndex="0" role="heading"></label>' ,
                    '              <span dojoAttachPoint="viewSubTitle" class="view-subtitle"></span>' ,
                    '              <span dojoAttachPoint="titleContainer" style="width:100%;">' ,                    
                    '                <span class="error-indicator dijit-reset error-decorate-image" dojoAttachPoint="fieldErrorNode" style="display:none" tabindex="-1"></span>' +                    
                    '				 <span role="heading">',
					'                    <span dojoAttachPoint="viewEditableTitle" tabIndex="0" dojoType="com.ibm.asq.common.web.ui.internal.widgets.TitleTextAreaEditor" display="inline" required="true"' ,
                    '                        value="${title}" style="max-width:100px" clearBorderAtStart=true class="view-title"></span>' ,
                    '                    <span dojoAttachPoint="dirtyIndicator" class="dirty-indicator" style="display: none">*</span>' ,
                    '				 </span>',
                    '                <span dojoAttachPoint="viewHelpButtonNode" class="help-button-node"></span>' ,
                    '              </span>' ,
                    '              <span dojoAttachPoint="viewTitleExt" style="display: none" class="view-title-ext"></span>' ,
                    '            </div>' ,
                    '          </div>' ,
                    '        </div>' ,
                    '      </div>' ,
                    '          <div style="clear:left; padding-top:4px">' ,
                    '            <span dojoAttachPoint="contentSelectorNode"class="view-artifact-type">' ,
                    '               <span dojoType="ViewHeaderLayout.InPageTabButton" dojoAttachPoint="mainContentSelector" dojoAttachEvent="onSelect:_selectMainContent"></span>',
                    '               <span dojoType="ViewHeaderLayout.InPageTabButton" dojoAttachPoint="altContentSelector" dojoAttachEvent="onSelect:_selectAltContent"></span>',
                    '               <span dojoType="ViewHeaderLayout.InPageTabButton" dojoAttachPoint="historyContentSelector" dojoAttachEvent="onSelect:_selectHistoryContent"></span>',                    
                    '            </span>' ,
                    '          </div>' ,
                    '      <div class="attributes-node" dojoAttachPoint="attributesContainerNode">' ,
                    '        <table style="width:100%;" role="presentation">' ,
                    '          <tr dojoAttachPoint="fullAttributeNode">' ,
                    '            <td style="width:100%;">' ,
                    '              <div dojoAttachPoint="attributesNode"></div>' ,
                    '            </td>' ,
                    '            <td>' ,
                    '              <div dojoAttachPoint="attributesNodeExt"></div>' ,
                    '            </td>' ,
                    '          </tr>' ,
                    '          <tr dojoAttachPoint="splitAttributeNode" style="display:none">' ,
                    '            <td style="width:50%;">' ,
                    '              <div dojoAttachPoint="splitAttributesNodeLeft" style="width:85%;"></div>' ,
                    '            </td>' ,
                    '            <td style="width:50%;">' ,
                    '              <div dojoAttachPoint="splitAttributesNodeRight" style="width:100%;"></div>' ,
                    '            </td>' ,
                    '          </tr>' ,
                    '			<tr>',
                    '				<div dojoAttachPoint="extraAttributesNode"></div>',	
                    '			</tr>',
                    '        </table>' ,
                    '      </div>' ,
                    '      <div dojoAttachPoint="summaryNode" class="summary-node" role="region">' ,
                    '        <label dojoAttachPoint="viewSummaryLabel" class="summary-label" style="float:left;" for="${id}-summary-text-area"></label>' ,
                    '        <span class="required-indicator" dojoAttachPoint="summaryRequiredNode" style="float:left;margin-right:2px;display:none" tabindex="-1">&#042;</span>' +
                    '        <span class="error-indicator dijit-reset error-decorate-image" dojoAttachPoint="summaryErrorNode" style="display:none" tabindex="-1"></span>' +
                    '        <span class="attributes-entry-value"></span>' ,
                    '        <span dojoAttachPoint="viewSummary" style="position:relative;top:3px;" class="attributes-entry-value"></span>' ,

                    // make the description/summary field the same tab order as the attributes.
                    '        <span dojoAttachPoint="viewEditableSummary" dojoType="com.ibm.asq.common.web.ui.internal.widgets.TitleTextAreaEditor" style="width:70%;" tabIndex="0"',
                    '              class="summary-value" id=${id}-summary-text-area clearBorderAtStart=true>' ,
                    '        </span>' ,
                    '      </div>' ,
                    '    </div>' ,
                    '  </div>' ,
                    '  <div dojoAttachPoint="containerNode" waiRole="region" class="view-container"></div>' ,
                    '  <div dojoAttachPoint="altContainerNode" waiRole="region" class="view-container" style="display:none"></div>' ,
                    '  <div dojoAttachPoint="historyContainerNode" waiRole="region" class="view-container" style="display:none"></div>' ,
                    '  <div dojoAttachPoint="approvalContainerNode" waiRole="region" class="view-container" style="display:none"></div>' ,
					'  <div dojoAttachPoint="viewHeaderBottomNode" class="view-header-container" style="display: none;" tabindex="-1">' ,
                    '      <div style="clear:right;width:100%;position:relative;">' ,
                    '         <div dojoAttachPoint="actionsBottomAreaRight" class="actions-area-right"></div>' ,
					'         <div dojoAttachPoint="actionsBottomAreaLeft" class="actions-area-left"></div>' ,
                    '      </div>' ,
                    '      <div style="clear:right;padding-left:5px;padding-top:3px">' ,
                    '        <div class="view-status-outter-container" dojoAttachPoint="wholestatusBottomContainer"> ' ,
                    '        <span dojoAttachPoint="statusBottomContainer" style="padding-right: 10px;">' ,
                    '          <span dojoType="jazz.ui.MessageBox" dojoAttachEvent="onClose:_onMessageClose" dojoAttachPoint="statusLineBottomNode"></span>' ,
                    '        </span>' ,
                    '        </div> ' ,
                    '      </div> ' ,
                    '  </div>' ,
                    '</div>'].join(""),

    vTabTemplateString: ['<div id="${id}" style="padding: 0px 4px" dojoAttachPoint="domNode" tabindex="-1" class="com-ibm-asq-common-web-ui-view-header">' ,
                    '  <div dojoAttachPoint="viewHeaderBackNode" class="view-header-back" tabindex="-1">' ,
                    '    <div class="view-header-container" tabindex="-1">' ,
                    '      <div dojoAttachPoint="breadcrumbContainerNode">' ,
                    '        <span dojoAttachPoint="breadcrumbNode" dojoType="jazz.ui.Breadcrumbs"></span>' ,
                    '      </div>' ,

                    '      <div class="view-status-outter-container" dojoAttachPoint="wholestatusContainer"> ' ,
                    '        <span dojoAttachPoint="statusContainer" role="status">' ,
                    '          <span dojoType="jazz.ui.MessageBox" dojoAttachPoint="statusLineNode" dojoAttachEvent="onClose:_onMessageClose" tabIndex="-1" aria-live="assertive"></span>' ,
                    '        </span>' ,
                    '      </div> ' ,

                    '      <div dojoAttachPoint="artifactContainer" class="artifact-container">' ,
                    '        <div dojoAttachPoint="iconNode" class="${iconClass}" style="float:left"></div>' ,
                    '        <div tabindex="-1" class="artifact-node">' ,
                    '          <div dojoAttachPoint="titleActionsContainer">' ,
                    '            <div dojoAttachPoint="actionsAreaRight" class="actions-area-right"></div>' ,
                    '            <div dojoAttachPoint="actionsAreaLeft" class="actions-area-left"></div>' ,
                    '            <div dojoAttachPoint="viewHeaderRight" waiRole="region" class="view-right-header" style="float:right"></div>' ,
                    '            <div dojoAttachPoint="progressIndicator" class="progress-indicator" style="display: none">' ,
                    '              <div class="view-progress-image">' ,
                    '                <span dojoAttachPoint="progressLabel" class="view-progress-label"></span>' ,
                    '              </div>' ,
                    '            </div>' ,
                    '            <div dojoAttachPoint="viewHeaderLeft"   class="view-left-header">' ,
                    '			  <span style="display:none" id=${id}_titleAriaLabel>${titleAriaLabel}</span>'+
                    '              <span class="required-indicator" dojoAttachPoint="fieldRequiredNode" style="float:left;margin:0px 2px;display:none" tabindex="-1">&#042;</span>' +
                    '              <label dojoAttachPoint="viewID" style="display:none;float:left;margin-right:2px;padding-top:4px;" class="view-title"></label>' ,
                    '              <span dojoAttachPoint="viewNonEditableTitle" class="view-title" tabIndex="0" style="display:none;" role="heading"></span>' ,                    
                    '              <span dojoAttachPoint="viewSubTitle" class="view-subtitle"></span>' ,
                    '              <span dojoAttachPoint="titleContainer" style="width:100%;">' ,                    
                    '                <span class="error-indicator dijit-reset error-decorate-image" dojoAttachPoint="fieldErrorNode" style="display:none" tabindex="-1"></span>' +                    
                    '				 <span role="heading">',
                    '                   <span dojoAttachPoint="viewEditableTitle" tabIndex="0" dojoType="com.ibm.asq.common.web.ui.internal.widgets.TitleTextAreaEditor" display="inline" required="true"' ,
                    '                        value="${title}" class="view-title" clearBorderAtStart=true ></span>' ,
                    '                </span>',
                    '                <span dojoAttachPoint="dirtyIndicator" class="dirty-indicator" style="display: none">*</span>' ,
                    '                <span dojoAttachPoint="viewHelpButtonNode" class="help-button-node"></span>' ,
                    '              </span>' ,
                    '              <span dojoAttachPoint="viewTitleExt" style="display: none" class="view-title-ext"></span>' ,
                    '            </div>' ,
                    '          </div>' ,
                    '        </div>' ,
                    '      </div>' ,
                    '    </div>' ,
                    '  </div>' ,

                    '  <table width="100%" style="table-layout:fixed;border-collapse:collapse;" role="presentation">',
                    '    <tr dojoAttachPoint="_layoutRow">',
                    '      <td valign="top" width="150px">',
                    '          <div dojoAttachPoint="vTabControlNode" style="clear:left;">' ,
                    '            <div dojoAttachPoint="topContentSelectorNode"></div>' ,

                    // NOTE: the tab buttons are not used in the vertical mode, but keep them in the template and hidden.
                    // The code still refers to them.
                    '            <div dojoAttachPoint="contentSelectorNode" class="view-artifact-type">' ,
                    '               <div dojoType="ViewHeaderLayout.InPageTabButton" dojoAttachPoint="mainContentSelector" dojoAttachEvent="onSelect:_selectMainContent"></div>',
                    '               <div dojoType="ViewHeaderLayout.InPageTabButton" dojoAttachPoint="altContentSelector" dojoAttachEvent="onSelect:_selectAltContent"></div>',
                    '               <div dojoType="ViewHeaderLayout.InPageTabButton" dojoAttachPoint="historyContentSelector" dojoAttachEvent="onSelect:_selectHistoryContent"></div>',                    
                    '            </div>' ,
                    '          </div>' ,
                    '      </td>',
                    '      <td valign="top" width="100%">',

                        '      <div class="attributes-node" dojoAttachPoint="attributesContainerNode" style="width:98%;">' ,
                        '        <table style="width:100%;" role="presentation">' ,
                        '          <tr dojoAttachPoint="fullAttributeNode">' ,
                        '            <td style="width:100%;">' ,
                        '              <div dojoAttachPoint="attributesNode"></div>' ,
                        '            </td>' ,
                        '            <td>' ,
                        '              <div dojoAttachPoint="attributesNodeExt"></div>' ,
                        '            </td>' ,
                        '          </tr>' ,
                        '          <tr dojoAttachPoint="splitAttributeNode" style="display:none">' ,
                        '            <td style="width:50%;">' ,
                        '              <div dojoAttachPoint="splitAttributesNodeLeft" style="width:85%;"></div>' ,
                        '            </td>' ,
                        '            <td style="width:50%;">' ,
                        '              <div dojoAttachPoint="splitAttributesNodeRight" style="width:100%;"></div>' ,
                        '            </td>' ,
                        '          </tr>' ,
                        '           <tr>',
                        '               <div dojoAttachPoint="extraAttributesNode"></div>', 
                        '           </tr>',
                        '        </table>' ,
                        '      </div>' ,
                        '      <div dojoAttachPoint="summaryNode" class="summary-node" style="width:98%;">' ,
                        '        <label dojoAttachPoint="viewSummaryLabel" class="summary-label" style="float:left;" for="${id}-summary-text-area"></label>' ,
                        '        <span class="required-indicator" dojoAttachPoint="summaryRequiredNode" style="float:left;margin-right:2px;display:none" tabindex="-1">&#042;</span>' +
                        '        <span class="error-indicator dijit-reset error-decorate-image" dojoAttachPoint="summaryErrorNode" style="display:none" tabindex="-1"></span>' +
                        '        <span class="attributes-entry-value"></span>' ,
                        '        <span dojoAttachPoint="viewSummary" style="position:relative;top:3px;" class="attributes-entry-value"></span>' ,
    
                        // make the description/summary field the same tab order as the attributes.
                        '        <span dojoAttachPoint="viewEditableSummary" dojoType="com.ibm.asq.common.web.ui.internal.widgets.TitleTextAreaEditor" style="width:70%;" tabIndex="0"',
                        '              class="summary-value" id=${id}-summary-text-area clearBorderAtStart=true>' ,
                        '        </span>' ,
                        '      </div>' ,


                    '        <div dojoAttachPoint="containerNode" waiRole="region" class="view-container"></div>' ,
                    '        <div dojoAttachPoint="altContainerNode" waiRole="region" class="view-container alternate-view-container" style="display:none"></div>' ,
                    '        <div dojoAttachPoint="historyContainerNode" waiRole="region" class="view-container alternate-view-container" style="display:none"></div>' ,
                    '        <div dojoAttachPoint="approvalContainerNode" waiRole="region" class="view-container alternate-view-container" style="display:none"></div>' ,
                    '      </td>',
                    '    </tr>',
                    '  </table>',

                    '  <div dojoAttachPoint="viewHeaderBottomNode" class="view-header-container" style="display: none;" tabindex="-1">' ,
                    '      <div style="clear:right;width:100%;position:relative;">' ,
                    '         <div dojoAttachPoint="actionsBottomAreaRight" class="actions-area-right"></div>' ,
                    '         <div dojoAttachPoint="actionsBottomAreaLeft" class="actions-area-left"></div>' ,
                    '      </div>' ,
                    '      <div style="clear:right;padding-left:5px;padding-top:3px">' ,
                    '        <div class="view-status-outter-container" dojoAttachPoint="wholestatusBottomContainer"> ' ,
                    '        <span dojoAttachPoint="statusBottomContainer" style="padding-right: 10px;">' ,
                    '          <span dojoType="jazz.ui.MessageBox" dojoAttachEvent="onClose:_onMessageClose" dojoAttachPoint="statusLineBottomNode"></span>' ,
                    '        </span>' ,
                    '        </div> ' ,
                    '      </div> ' ,
                    '  </div>' ,
                    '</div>'].join(""),

    isEditor: true,
    titleEditable: false,
    contextHelpId: null,
	cshelp: null,
    iconClass: null,
    historyActivated: false,
    approvalActivated: false,
    noTitleLabel: "",  // prompt label for editable title when no value is specified
    noSummaryLabel: "",  // prompt label for editable summary when no value is specified
    summaryAsDesc: true, // The summary node is mostly used for description so the no value prompt
                         // string is default to say description.  Set this to false to have the
                         // default no value prompt string to say summary.
    title: "",
	maxTitleLength: 250,  // Maximum number of characters that can be entered in the editable title in line edit box
	maxSummaryLength: 0,  // Maximum number of characters that can be entered in the summary, 0 for no limit
	subTitle: "",
    typeLabel: "",
    summaryEditable: false,
    summaryLabel: "",
    summary: "",
    noline: false,
    titleDisplayStyle: _dojo.isIE ? "inline-block" : "inline", // inline style allow wrapping if title is too long that ruuns into the floating actions
    
    altContentMarkerEnabled:true,
    useDualContents: false,
    defaultMainContent: true,
    // consumer should specify these labels
    //
    mainContentSelectedLabel: "",
    mainContentDeselectedLabel: "",
    altContentSelectedLabel: "",
    altContentDeselectedLabel: "",
    historyContentSelectedLabel: "",
    historyContentDeselectedLabel: "",         
    
    widgetsInTemplate: true,
    statusPopup: null,
	uninitialized: false,
		
	isDisplayBottomButton: true,
    showBreadcrumb: false,
	
    titleAriaLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelTitle"),
	// Do not change this attribute since it can create undesire extra space on the Layout 
	minHeight : 0,  
    
    // if in compact mode, the view header removes the attribute and 
    // the summary areas and also reduces the content container's top
    // padding. Compact mode is for pages that don't need the mentioned
    // areas and the associated empty height associated with them.
    isCompact: false,
    
    isMinMarginLeft: false, 
    
    // header messages are shown in a message box that can be closed by the user. By Default,
    // only SUCCESS type messages are closable by the user. Specify true in this attribute
    // will allow INFO type to be closable by the user as well. All other message types can't
    // be closed by the user.
    infoStatusClosable: false,
    
    // if the title is editable and it's required as indicated by this flag,
    // an error indicator will be shown if the title is empty.
    titleIsRequired: true,
    summaryIsRequired: false,
    msgNameIsRequired: MessageUtil.getMessage(MessageBundle.COMMON, "msgNameIsRequired"),
    msgSummaryIsRequired: MessageUtil.getMessage(MessageBundle.COMMON, "msgSummaryIsRequired"),
    validationStarted: false,
    msgNameEditAltText: MessageUtil.getMessage(MessageBundle.COMMON, "labelTitle"),
    msgDescriptionEditAltText: MessageUtil.getMessage(MessageBundle.COMMON, "labeldescription"),
    
    // This attribute is only settable during creation. When set to true, the InfoAttributes area will  
    // split into two equal columns. The 'normal' added InfoAttributes will be placed on the left column
    // and their labels & values are properly aligned. The right area allows the client to add additional
    // free form content. If this mode is used the InforAttributeExt is not supported.
    //
    splitAttributeArea: false,
    attributeFieldWidth: 220,
    
    _trimStatusTextMax: TRIM_MAX,
    
    // display the content control tabs horizontally (default), or vertically on the side. When 
    // this attribute is set to 'vertical', no tab buttons would be shown. It's up to the client
    // to add the control into the vertical selector node, 'topContentSelectorNode'. The client still
    // sets up the alternate contents with the header and use the change content methods to switch
    // contents.
    //
    tabsControlOrientation: 'horizontal',
    
    postMixInProperties: function(){
        this.templateString = this.tabsControlOrientation == 'horizontal' ? this.hTabTemplateString : this.vTabTemplateString;
        this.inherited('postMixInProperties',arguments);
    },
    
    postCreate: function() {
        this.inherited('postCreate', arguments);
        
        this.dirty = false;
        
		//Set the max characters & absolute width hints on the editable title
		this.viewEditableTitle.maxLength = this.maxTitleLength;
		this.viewEditableTitle.maxTextAreaLength = this.maxTitleLength;
        
        //Set the max characters & absolute width hints  on the editable summary
        this.viewEditableSummary.maxTextAreaLength = this.maxSummaryLength;
        
        // server return status in hidden until a status is set
        //
        //this.statusContainer.style.display = "none";
		//bottom button area
		if(this.isDisplayBottomButton) {
			this.viewHeaderBottomNode.style.display = "";
			//this.statusBottomContainer.style.display = "none";
		}		        
        if(this.titleEditable){
        	this.setNoValueEditableTitleLabel(this.noTitleLabel);
        }
        this.setNoValueEditableSummaryLabel(this.noSummaryLabel);
        
        this.setTitleEditable(this.titleEditable);
        this.setSubTitle(this.subTitle);
        this.setTitle(this.title);        
        
        this.setTypeLabel(this.typeLabel);
        this.setIconClass(this.iconClass);
        this.setCSHelpId(this.contextHelpId);

        this.setSummaryEditable(this.summaryEditable);
        this.setSummaryLabel(this.summaryLabel);
        this.setSummary(this.summary);

        // set up nodes for dual contents support
        this.contentSelectorNode.style.display = (this.tabsControlOrientation == 'horizontal' && (this.useDualContents || this.historyActivated || this.approvalActivated)) ? "inline" : "none";

        var inPageTabsGroup = [];
        if (this.useDualContents || this.historyActivated || this.approvalActivated) {
            this.mainContentSelector.set('group', inPageTabsGroup);
            if (this.mainContentSelectedLabel == "")
                this.mainContentSelectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderMainContentSelected");
            if (this.mainContentDeselectedLabel == "")
                this.mainContentDeselectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderMainContentDeselected");
            if(this.altContentMarkerEnabled){
                if (this.altContentSelectedLabel == "")
                    this.altContentSelectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderAltContentSelected");
                if (this.altContentDeselectedLabel == "")
                    this.altContentDeselectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderAltContentDeselected");
                this.altContentSelector.set('group', inPageTabsGroup);
            }
            else{
                this.altContentSelectedLabel == "";
                this.altContentDeselectedLabel == "";
                this.contentSelectorNode.removeChild(this.altContentSelector.domNode);     
            }
		    if (this.historyContentDeselectedLabel == "")
                this.historyContentDeselectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderHistoryContentDeselected");
		    
		    if (this.defaultMainContent) {
                this._selectMainContent();
                this.mainContentSelector.set('selected', true);
            } else {
                this._selectAltContent();
                this.altContentSelector.set('selected', true);
            }
        }
        if (this.historyActivated) {
            this.historyContentSelector.set('group', inPageTabsGroup);
        } else {
        	//If history was not activated yet, hide the history tab
        	this.historyContentSelector.style.display = "none";
        }
        
        if( this.noline )
        {
        	this.viewHeaderBackNode.className = "view-header-back-no-line";
        }
		
		//set icon node to the right in RTL
		var lang = dojo.locale.split('-')[0];
		var RTL = {ar:true,fa:true,he:true,ur:true,yi:true,iw:true};
		if(RTL[lang]) {
			this.iconNode.style.cssFloat = "right";
			this.actionsAreaRight.style.cssFloat = "right";
			this.actionsAreaLeft.style.cssFloat = "right";
			this.viewHeaderRight.style.cssFloat = "right";
			this.wholestatusContainer.style.cssFloat = "left";
			this.statusContainer.style.cssFloat = "right";
			this.viewSummaryLabel.style.cssFloat = "right";
			
			this.actionsBottomAreaRight.style.cssFloat = "right";
			this.actionsBottomAreaLeft.style.cssFloat = "right";
			this.wholestatusBottomContainer.style.cssFloat = "left";
			this.statusBottomContainer.style.cssFloat = "right";
		}
		
		if (!dojo.isIE) {
			this.wholestatusBottomContainer.style.clear = "right";
		}
        
        // store the view's right buttons for release later
        this.buttons = [];
        
        this.setHeight();
        
        // compact mode to reduce the height of the empty area
        if(this.isCompact) {
            dojo.style(this.attributesContainerNode, "display", "none");
            dojo.style(this.summaryNode, "display", "none");
            dojo.addClass(this.containerNode, "view-container-compact");
        }
        if(this.isMinMarginLeft) {
            dojo.addClass(this.containerNode, "view-container-compact-min-margin-left");
            dojo.addClass(this.viewHeaderBackNode, "view-header-compact-min-margin-left");
        }
        
        dojo.style(this.breadcrumbContainerNode, "display", this.showBreadcrumb ? "" : "none");
        
        // split the info attributes area into two columns.
        // use the split table row.
        if(this.splitAttributeArea) {
            dojo.style(this.fullAttributeNode, "display", "none");
            dojo.style(this.splitAttributeNode, "display", "");
            
            // create a group object o align the InfoAttributes' labels.
            this.attributesNode = this.splitAttributesNodeLeft;
            this.attributesGroup = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormFieldGroup({
                fieldWidth: this.attributeFieldWidth
            });
            dojo.place(this.attributesGroup.domNode, this.splitAttributesNodeLeft);
        }
        this.viewEditableTitle.set("alt", this.msgNameEditAltText);
        this.viewEditableTitle.domNode.setAttribute("aria-labelledby",this.id + "_titleAriaLabel");
        this.viewEditableSummary.set("alt", this.msgDescriptionEditAltText);
        //to force JAWS to announce the Status Message.
        dojo.attr(this.statusLineNode.messageArea,{
        	"role":"alert",
        	"aria-live":"assertive"
        });
        
        //subscribe to event published after Globalization preferences are changed
        this._handleBidiChange = dojo.subscribe(BidiUtil.BIDI_CHANGED, this, "setTextDir");
        this.setTextDir();
          
    },
	
    // Set text direction property, apply text direction on the widgets, fix text alignment
    setTextDir: function() { 	
    	var textDir = BidiUtil.getWidgetBaseTextDir();
    	if (!this.textDir && !textDir) return;
    	this.textDir = textDir; //also for case, that bidi support is changed to "off"
    	var orientation = this.isLeftToRight() ? "left" : "right";
    	if (this.viewEditableTitle) this.viewEditableTitle.set('textDir', this.textDir);
    	if (this.viewEditableSummary) {
    		this.viewEditableSummary.set('textDir', this.textDir);
    		if (this.textDir) dojo.style(this.viewEditableSummary.domNode, "textAlign", orientation); 
    	}
    	if (this.viewNonEditableTitle) BidiUtil.applyTextDirSetting(this, this.viewNonEditableTitle, this.viewNonEditableTitle.innerHTML);
        if (this.viewSummary) BidiUtil.applyTextDirSetting(this, this.viewSummary, this.viewSummary.innerHTML);
     },        

    // resize the width of the content selector area.
    setContentSelectorWidth: function(width) {
        dojo.style(this.vTabControlNode.parentNode, 'width', width+'px');
    },
    
    setEditableTitleFocus: function()
    {
        // delay setting the editable title focus to let the widget load.
        setTimeout(dojo.hitch(this, function() {
        	this.viewEditableTitle.focus();
        }), 30);
    },
    
    // set the no value prompt label for editable title
    //
    setNoValueEditableTitleLabel: function(label) {    	
		var viewEditableTitle = this.viewEditableTitle;
		this.noTitleLabel = label;
        if (!label) {
			label = MessageUtil.getMessage(MessageBundle.COMMON, "promptLabel_clickToEdit");
        }	
        if(viewEditableTitle && this.titleEditable){
        	viewEditableTitle.setNoValuePrompt(label);
        }		
    },
    
    // change handler for editable title
    //
    editableTitleChange: function(value) {
		this.title = value;
        this._updateTitleRequiredError();
        this.onTitleEdited(value);
    },

	calculateMaxWidth: function() {
		
		var iconNodeWidth = 0;
		var fieldRequiredNodeWidth = 0;
		var viewIDWidth = 0;
		var viewHelpButtonNodeWidth = 0;
				
		var isIconNodeShown = (dojo.style(this.iconNode, 'display') != 'none');
		if(isIconNodeShown) {
			iconNodeWidth = dojo.marginBox(this.iconNode).w;
		}
		
		var isFieldRequiredShown = (dojo.style(this.fieldRequiredNode, 'display') != 'none');
		if(isFieldRequiredShown) {
			fieldRequiredNodeWidth = dojo.marginBox(this.fieldRequiredNode).w;
		}
		
		var isViewIDShown = (dojo.style(this.viewID, 'display') != 'none');
		if(isViewIDShown) {
			viewIDWidth = dojo.marginBox(this.viewID).w;
		}
		
		var isHelpButtonShown = (dojo.style(this.viewHelpButtonNode, 'display') != 'none');
		if(isHelpButtonShown) {			
			viewHelpButtonNodeWidth = dojo.marginBox(this.viewHelpButtonNode).w;
		}
		
		//offset contains dirty indicator(not always shown) + paddings + an extra space
		var offset = dojo.isIE ? 55 : 45;		
		
		var maxWidth = (this.viewHeaderRight.offsetLeft) - (iconNodeWidth + fieldRequiredNodeWidth + viewIDWidth + viewHelpButtonNodeWidth) - offset; 		       
		return maxWidth;
	},
    
	// get the precise width in pixels of the title using the word wrapping util
	//
    _getTitleWidth: function(title) {
        // specify style for the word wrapping util in case the widget is not displayed yet and the class is not loaded
        // must match the class "com-ibm-asq-common-web-ui-view-header view-title"
        var style = {
            fontFamily: "Arial, san-serif",
            fontSize: "14pt",
            fontWeight: "normal"
        };
        var len = wordWrapping.getTextWidth(Utility.encodeHTMLTags(title), this.viewEditableTitle.className, style);
        // add the 10px margin and some padding that the word wrapping util does not include
        return len + 16;
    },
    
	showEditableTitle: function() {
		var editableTitleWidth = this.calculateMaxWidth();
		dojo.style(this.viewEditableTitle.domNode, "width", editableTitleWidth + "px");				
		dojo.style(this.viewEditableTitle.domNode, "maxWidth", "" );
	},

	restoreWidth: function(e){
		var val = this.calculateMaxWidth();
		dojo.style(this.viewEditableTitle.domNode, "width", "");
		dojo.style(this.viewEditableTitle.domNode, "maxWidth", val + "px");
		var textContent = this.viewEditableTitle.getValue();					
		if(!textContent || textContent == "" ) {
		    textContent = this.noTitleLabel;
		}

		var len = this._getTitleWidth(textContent);
		dojo.style(this.viewEditableTitle.domNode,"width",len +"px");
		this.viewEditableTitle.resize(); //resize vertically too
	},
	
	setTitleEditable: function(editable) {
		this.titleEditable = editable;                    
		this.viewEditableTitle.set('disabled',!editable);
		if(editable){			
			this.connect(this.viewEditableTitle.domNode, "onfocus", "showEditableTitle");
			this.connect(this.viewEditableTitle.domNode, "onblur", "restoreWidth");
			this.connect(this.viewEditableTitle, "valueChanged", "editableTitleChange");

			// only show the required title decoration if the title is editable & is required
			dojo.style(this.fieldRequiredNode, "display", (this.titleIsRequired) ? "" : "none");
	
			dojo.style(this.viewEditableTitle.domNode,"display","inline");
			dojo.style(this.viewNonEditableTitle,"display","none");			
			dojo.style(this.viewID,"float","left");
		} else {
			dojo.style(this.viewID,"float","");
			dojo.style(this.viewEditableTitle.domNode,"display","none");
			dojo.style(this.viewNonEditableTitle,"display","inline");
		}
		
		if(this.viewSubTitle){
			dojo.style(this.viewSubTitle, "display",this.titleEditable ? "none" : "inline" );        	
		}

		//Scrollbar may be shown occasionally. Hide it
		if(this.viewEditableTitle) {
			this.viewEditableTitle.domNode.style.overflowX = "hidden";
		}

		if (editable) {
    		// register to get notification from window resize events if this is editable to adjust the title max width
		    if (!this.resizeNotifier) {
                this.resizeNotifier = jazz.util.ResizeNotifier.register({context: this, callback: "_resizeHandler"});
		    }
		} else {
		    // unregister if this is changed to be non-editable
            if (this.resizeNotifier) {
                jazz.util.ResizeNotifier.unregister(this.resizeNotifier);
                delete this.resizeNotifier;
            }
		}
	},
    
	// Resize handler to adjust the width of the title on window resized
	//
    _resizeHandler: function() {
        if (dojo.hasClass(this.viewEditableTitle.domNode, "title-clearborder")) {
            // not in edit mode
            this.restoreWidth();
        } else {
            // editable title is already shown, invoke again to recalculate the width
            this.showEditableTitle();
        }
    },
    
    setDirty: function(dirty) {
        this.dirty = dirty;
        dojo.style(this.dirtyIndicator, "display", dirty ? "inline" : "none");        
    },

    // Provide a more meaningful ARIA text for the title if the view is marked dirty. 
    _updateAllyLabel: function() {
        var val = this.viewEditableTitle.getValue();
        var allyLabel = (this.dirty) ? MessageUtil.getMessage(MessageBundle.COMMON, "labelAllyModified", val) : val;
        dijit.setWaiState(this.viewEditableTitle, "label", allyLabel);        
    },
    
	uninitialize: function() {
		//console.debug("========uninitialize ViewHeaderLayout");
		this.uninitialized = true;
		var viewEditableTitle = this.viewEditableTitle;
		if (viewEditableTitle) {
			viewEditableTitle.onChange = null;
		}
		var viewEditableSummary = this.viewEditableSummary;
		if (viewEditableSummary) {
			viewEditableSummary.onChange = null;
		}
		if(this.cshelp)
			this.cshelp.destroy();
        if(this.buttons) {
            dojo.forEach(this.buttons, function(pairButton){
               // If the buttons are just plain dom buttons,
               // need to clear its ref to the JS data.
               if(!pairButton.isButtonWidget) {
                   pairButton.headerButton.clientCallback = null;
                   pairButton.headerButton.callbackData = null;
                   pairButton.bottomButton.clientCallback = null;
                   pairButton.bottomButton.callbackData = null;
               } 
            });
        }
        if (this.statusPopup) {
            this.statusPopup.destroy();
        }
        if (this.statusBottomPopup) {
            this.statusBottomPopup.destroy();
        }
        if (this.dummyText) {
            this.dummyText.destroy();
        }
        if(this.attributesGroup) {
            this.attributesGroup.destroyRecursive();
        }
        if (this.resizeNotifier) {
            jazz.util.ResizeNotifier.unregister(this.resizeNotifier);
            delete this.resizeNotifier;
        }
		if (this._handleBidiChange) {
			dojo.unsubscribe(this._handleBidiChange);
			this._handleBidiChange = null;
		}    
		if (this.altContentSelector) {
			// the domnode for this widget may have been removed fromt the tree, so the default cleanup won't find it. 
			this.altContentSelector.destroyRecursive();
			this.altContentSelector=null;
		}

	},	
	
    // set the title to display in the header
    //
    setTitle: function(title) {    	
        this.title = title;    
        
        //Set the viewEditableTitle/viewNonEditableTitle properties since the titleEditable property may not be set:
        this.viewEditableTitle.setValue(title);    
		
		//Note: Encode the HTML tags in the title.
		this.viewNonEditableTitle.innerHTML = Utility.encodeHTMLTags(title);    	

        if (this.titleEditable) {
            var len = this._getTitleWidth(title);
        	dojo.style(this.viewEditableTitle.domNode,"width",len +"px");	
            this._updateTitleRequiredError();
            //If tex  is very large we should force  an auto fit
            setTimeout(dojo.hitch(this,"restoreWidth"), 0);            
        }
        else {
    		this.setTitleEditable(false);    	
        }
        
    },
    
    // show/hide the title's field required-ness error status based on the value.
    _updateTitleRequiredError: function(){
    	if (this.validationStarted) {
	        var title = this.title;
	        if (this.titleIsRequired) {
	            dojo.style(this.fieldErrorNode, "display", !title ? "" : "none");
	            this.fieldErrorNode.title = !title ? this.msgNameIsRequired : "";
	        }
    	}
    },
    
    // set the title to display in the header
    //
    setID: function(id) {
		this.viewId = id;
        if (id && id != "" && !isNaN(id)) {
			if(this.viewID){
            	dojo.style(this.viewID, "display", "inline");
            	this.viewID.innerHTML = MessageUtil.getMessage(MessageBundle.COMMON, "labelIdLabel", id);
			}
        }       
    },
	
    setSubTitle: function(subTitle) {
    	this.subTitle = Utility.encodeHTMLTags(subTitle);
    	if (this.viewSubTitle){
    		this.viewSubTitle.innerHTML = this.subTitle ? this.subTitle : "";
    	}
    },

    setTitleExt: function(extension) {
        if (this.viewTitleExt) {
            dojo.style(this.viewTitleExt, "display", "");
            this.viewTitleExt.innerHTML = extension ? "&nbsp;" + extension : "";
        }
    },
    
    setBreadcrumb: function(crumbs) {
        if (this.breadcrumbContainerNode) {
            var hasCrumb = dojo.isArray(crumbs) && (crumbs.length > 0);
            dojo.style(this.breadcrumbContainerNode, "display", hasCrumb ? "" : "none");
            if (hasCrumb) {
                this.breadcrumbNode.segments = crumbs;
                this.breadcrumbNode.refresh();
            }
        }
    },
    
    getTitle: function() { 
    	return this.title;
    },
    
    setTypeLabel: function(type) {
        this.typeLabel = type;
    },
    
    setIconClass: function(iconClass) {
        this.iconClass = iconClass;
        if (this.iconNode) {
            this.iconNode.className = this.iconClass ? this.iconClass : "";
            this.iconNode.style.paddingRight = "2px";
        }
		if(this.iconClass || this.isMinMarginLeft)
			this.viewHeaderLeft.style.paddingLeft = "0px";
		else
			this.viewHeaderLeft.style.paddingLeft = "10px";
    },
    
    setCSHelpId: function(helpId){
        this.contextHelpId = helpId;
        if (this.viewHelpButtonNode) {
            dojox.xml.parser.removeChildren(this.viewHelpButtonNode);
            if (this.contextHelpId) {
                this.viewHelpButtonNode.style.display = "inline";
                var args = {
                    'contextID': helpId
                };
				if(this.cshelp)
					this.cshelp.destroy();
                this.cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp(args);
                this.viewHelpButtonNode.appendChild(this.cshelp.domNode);
            } else {
                this.viewHelpButtonNode.style.display = "none";
            }
        }
    },
    
   
	setCSHelpButtonLabel: function(/* REQUIRED */ cshelpLabel) {
		if(this.cshelp)
			this.cshelp.setButtonLabel(cshelpLabel);
	},
    // set the no value prompt label for editable title
    //
    setNoValueEditableSummaryLabel: function(label) {
        this.noSummaryLabel = label;
        if (!label) {
            if (this.summaryAsDesc) 
                label = MessageUtil.getMessage(MessageBundle.COMMON, "promptLabel_clickEnterDesc");
            else 
                label = MessageUtil.getMessage(MessageBundle.COMMON, "promptLabel_clickEnterSummary");
        }
        this.viewEditableSummary.setNoValuePrompt(label);
    },
    
    setSummaryEditable: function(editable) {
        this.summaryEditable = editable;
        this.viewEditableSummary.domNode.style.display = this.summaryEditable ? "inline" : "none";
        this.viewSummary.style.display = this.summaryEditable ? "none" : "inline";
        dojo.style(this.summaryRequiredNode, "display", (this.summaryIsRequired && this.summaryEditable) ? "" : "none");        
        if (this.summaryEditable) {
            this.connect(this.viewEditableSummary, "valueChanged", "editableSummaryChange");
        }
    },

    // show/hide the summary's field required-ness error status based on the value.
    _updateSummaryRequiredError: function() {
    	if (this.validationStarted) {
	        var value = this.summary;
	        if(this.summaryIsRequired) {
	            dojo.style(this.summaryErrorNode, "display", !value ? "" : "none");
	            this.summaryErrorNode.title = !value ? this.msgSummaryIsRequired : "";
	        }
    	}
    },
    
    editableSummaryChange: function(value) {
        this.summary = value;
        this._updateSummaryRequiredError();
        this.onSummaryEdited(value);
    },
    
    setSummaryLabel: function(label) {
        this.summaryLabel = label;
        this.viewSummaryLabel.innerHTML = label ? Utility.encodeHTMLTags(label) : "";
        this.viewSummaryLabel.style.display = label ? "inline" : "none";
    },
    
    setSummary: function(summary, stayEdit) {
        this.summary = summary;
        if (this.viewSummary) {
            this.viewSummary.innerHTML = this.summary ? Utility.encodeHTMLTags(this.summary) : "";
        }
        if (this.viewEditableSummary) {
            this.viewEditableSummary.setValue(this.summary);
            this._updateSummaryRequiredError();
        }
    },
    
    getSummary: function() {
        this.summary = this.viewEditableSummary.getValue();
        return this.summary;
    },
    
    // Summary:
    //    set the status message in the header. Optionally, provide a type "Success", "Error",
    //    or "Info" parameter to qualify the status. 
    //
    //    Optional details - provide a detailed message in addition to the status text.
    //    Optional errorObj - provide a server returned error object to be displayed.
    //
    setStatus: function(status, type, /*optional*/details, /*optional*/errorObj) {
        
        type = type || ViewHeaderLayout.INFO;
        var stType = jazz.ui.MessageBox.STATUS_TYPES.INFO;
        if (type == ViewHeaderLayout.SUCCESS) {
            stType = jazz.ui.MessageBox.STATUS_TYPES.OK;
        } else if (type == ViewHeaderLayout.ERROR) {
            stType = jazz.ui.MessageBox.STATUS_TYPES.ERROR;
        } else if (type == ViewHeaderLayout.WARNING) {
            stType = jazz.ui.MessageBox.STATUS_TYPES.WARNING;
        } 
        var stText = status;
        if ((status.length - TRIM_MAX) > 5) {
            stText = status.substr(0, TRIM_MAX) + "...";
            if(!details) {
                details = status;
            }
        }
        var detailsRec = details ? [{message:details, code:'OK'}] : null;
        var messageArgs = {
            summary: stText,
            type: stType,
            details: detailsRec,
            // always allow SUCCESS type to be closable and if INFO type is allow closable, allow that too
            closable: (type == ViewHeaderLayout.SUCCESS || (this.infoStatusClosable && type == ViewHeaderLayout.INFO))
        };
        if(errorObj) {
            messageArgs.errorObject = errorObj;
        }
        setTimeout(dojo.hitch(this,function(){
            // if for some reason widget destruction was started
            // while we were waiting to update skip update
            if (this._beingDestroyed) return;
            if(this.statusLineNode){
            	this.statusLineNode.showMessage(messageArgs);
            	this.statusLineBottomNode.showMessage(messageArgs);
            	this.statusClosed = false;
            }
        }), 500);        
    },
	
    // handles the message box being closed by the user.
    _onMessageClose: function(){
        // close both statuses line, if the user closes 1.
        this.clearStatus();
    },
    
    // set the status message in the header
    //
    clearStatus: function() {
        this.statusLineNode.hideMessage();
        this.statusLineBottomNode.hideMessage();
        this.statusClosed = true;
    },

    // Summary:
    //    show, or hide the content area DIV of the ViewHeaderLayout. 
    // 
    showContentArea: function(showContent /* true to show, false to hide */) {
        this.containerNode.style.display = (showContent) ? "inline" : "none";
    },
    
    // Summary:
    //    miximize or minimize the content area DIV of the ViewHeaderLayout. 
    // 
    maximizeContentArea: function(maximize /* true to maximize, false to minimize */, hideButtons/* *Optional* hides/restores buttons in the main editor toolbar */){
    	var _dojo = dojo;
    	var displayStyle = maximize ? "none" : "";
		_dojo.style(this.summaryNode, "display", displayStyle);
		_dojo.style(this.attributesContainerNode, "display", displayStyle);
		
		// horizontal layout style doesn't hae the vtab control. check for it
		if(this.vTabControlNode) {
		    _dojo.style(this.vTabControlNode.parentNode, "display", displayStyle);
		}
		
		var displayStyleForToolbars = maximize ? "none" : "block";
		if(hideButtons){
			_dojo.style(this.actionsAreaLeft, "display", displayStyleForToolbars);
			_dojo.style(this.actionsBottomAreaLeft, "display", displayStyleForToolbars);			
		}
		
    },
    
    // show the in-progress indicator and hide the title. Optionally, specify
    // whether the progress should hide the view's contents. If not specified, always shows
    // all contents. By hiding the view's content while the progress is showing,
    // this reduces the flickering while content is being loaded. When the progress is
    // ended all hidden content are shown.
    //
    setProgressStart: function(message, hideContent) {
        if (hideContent) {
            _dojo.style(this.attributesContainerNode, "visibility", "hidden");
            _dojo.style(this.contentSelectorNode, "visibility", "hidden");
            _dojo.style(this.summaryNode, "visibility", "hidden");
            _dojo.style(this.actionsAreaRight, "visibility", "hidden");
            _dojo.style(this.actionsAreaLeft, "visibility", "hidden");
            _dojo.style(this.wholestatusContainer, "visibility", "hidden");
            
            this.progressHideContent = true;
			
			//bottom area
			_dojo.style(this.actionsBottomAreaRight, "visibility", "hidden");
            _dojo.style(this.wholestatusBottomContainer, "visibility", "hidden");
			_dojo.style(this.actionsBottomAreaLeft, "visibility", "hidden");
        }
        this.progressLabel.innerHTML = message;
        _dojo.style(this.viewHeaderLeft, "display", "none");
        _dojo.style(this.progressIndicator, "display", "inline");
    },
    
    // restore the title and hide the in-progress indicator
    //
    setProgressEnd: function() {
        if(this.progressHideContent) {
            _dojo.style(this.attributesContainerNode, "visibility", "visible");
            _dojo.style(this.contentSelectorNode, "visibility", "visible");
            _dojo.style(this.summaryNode, "visibility", "visible");
            _dojo.style(this.actionsAreaRight, "visibility", "visible");
            _dojo.style(this.actionsAreaLeft, "visibility", "visible");
            _dojo.style(this.wholestatusContainer, "visibility", "visible");
			
			//bottom area
			_dojo.style(this.actionsBottomAreaRight, "visibility", "visible");
            _dojo.style(this.wholestatusBottomContainer, "visibility", "visible");
			_dojo.style(this.actionsBottomAreaLeft, "visibility", "visible");
			
            this.progressHideContent = false;
        }
        if(this.progressLabel){
        	this.progressLabel.innerHTML = "";
        }
        if(this.viewHeaderLeft){
        	_dojo.style(this.viewHeaderLeft, "display", this.titleDisplayStyle);
        }
        if(this.progressIndicator){
        	_dojo.style(this.progressIndicator, "display", "none");
        }
    },

    _addViewAction: function(label, handler, callbackData, iconClass, left, showActionLabel, dropDown, pos) {
        var button = null;
		var arg = {
            label: label,
            showLabel: showActionLabel,
            'class': left ? "left-action" : "right-action",
            onClick: function( e ) {
                this.clientCallback(this.callbackData, e);
            }
        };
        
        if (iconClass) {
            arg["iconClass"] = iconClass;
        }
		if(dropDown) {
			arg['dropDown'] = dropDown;
			button = new com.ibm.asq.common.web.ui.internal.widgets.MenuButton(arg);
		}
		else 
        	button = new com.ibm.asq.common.web.ui.internal.widgets.Button(arg);
        
        // store the client callback and data into the button
        button.clientCallback = handler;
        button.callbackData = callbackData;
        
       var placePos = pos ? pos : "last";
        if (left) {
        	dojo.place(button.domNode, this.getleftActionsContainer(), placePos);
        }
        else {
        	dojo.place(button.domNode, this.getRightActionsContainer(), placePos);
        }
            
        
		//bottom button
		var bottomButton = null;
		if(dropDown) {
			bottomButton = new com.ibm.asq.common.web.ui.internal.widgets.MenuButton(arg);
		}
		else 
        	bottomButton = new com.ibm.asq.common.web.ui.internal.widgets.Button(arg);
		bottomButton.clientCallback = handler;
        bottomButton.callbackData = callbackData;
		if (left) {
        	dojo.place(bottomButton.domNode, this.getleftActionsBottomContainer(), placePos);
        }
        else {
        	dojo.place(bottomButton.domNode, this.getRightActionsBottomContainer(), placePos);
        }
		
		var pairButtons = new HeaderLayout.PairButtons(button, bottomButton, true);	
        return pairButtons;
    },

     // Specify a pos value to place the button in a particular position in the tool bar. Use
    // "first", "last" or a numeric value for the position. By default, "last" is the placement
    // position.
    //
    addLeftViewAction: function(label, handler, callbackData, iconClass, dropDown, pos) {
        this.actionsAreaLeft.style.display = "block";
		this.actionsBottomAreaLeft.style.display = "block";
        return this._addViewAction(label, handler, callbackData, iconClass, true, false, dropDown, pos);
    },
    
    addRightViewAction: function(label, handler, callbackData, iconClass, buttonId) {
        this.actionsAreaRight.style.display = "block";
        this.actionsBottomAreaRight.style.display = "block";

//  use plain button at the moment
        var button = document.createElement('button');
        button.innerHTML = label;
        button.title = label;
        button.className = "right-action";
        if(buttonId && buttonId!= ""){
        	button.setAttribute("id", buttonId);
        }
        this.connect(button, "onclick",  function( e ) { 
            e = e || window.event;
            e.target.focus();
            e.target.clientCallback(e.target.callbackData, e); 
		});
		this.connect(button, "onmousedown", dojo.stopEvent);
        button.tabIndex = "0";
		
        // store the client callback and data into the button
        // can't use dijit.connect for these as no dontFix arg is supported?
        button.clientCallback = handler;
        button.callbackData = callbackData;
        this.getRightActionsContainer().appendChild(button);
		
		//bottom area
        var bottomButton = button.cloneNode(true);
        
        // the bottom button should only be in the tab navigation list, but not the same as the
        // top buttons. 
        bottomButton.tabIndex = "0";
		this.connect(bottomButton, "onclick", function( e ) { 
            e = e || window.event;
            e.target.focus();
            e.target.clientCallback(e.target.callbackData, e); 
		});
		bottomButton.clientCallback = handler;
        bottomButton.callbackData = callbackData;
		this.connect(bottomButton, "onmousedown", dojo.stopEvent);
        this.getRightActionsBottomContainer().appendChild(bottomButton);
		
		var pairButtons = new HeaderLayout.PairButtons(button, bottomButton, false);
        
        // save the buttons, need to uninitialize the dom buttons
        // when the widget is destroy to break any circular refs.
        this.buttons.push(pairButtons);
        	
        return pairButtons;	
    },
    
    // return the right header DIV
    //
    getRightHeader: function() {
        return this.viewHeaderRight;
    },
    
    // return the main content area DIV
    //
    getContentNode: function() {
        return this.containerNode;
    },
    
    // return the alternate content area DIV
    //
    getAltContentNode: function() {
        return this.altContainerNode;
    },
        
    // return the history content area DIV
    //
    getHistoryContentNode: function() {
        return this.historyContainerNode;
    },
    
    
    _selectMainContent: function() {
    	if (this.useDualContents || this.historyActivated || this.approvalActivated) {
    		this.defaultMainContent = true;
    		
    		if (this.containerNode) {
    			this.mainContentSelector.set('label', this.mainContentSelectedLabel);
    			this.containerNode.style.display = "block";
    		}            
    		if(this.useDualContents && this.altContainerNode){
    			this.altContentSelector.set('label', this.altContentDeselectedLabel);
    			this.altContainerNode.style.display = "none";
    		}
    		if(this.historyActivated && this.historyContainerNode){
    			this.historyContentSelector.set('label', this.historyContentDeselectedLabel);
    			this.historyContainerNode.style.display = "none";
    		}
    		//set which content is selected
    		this._mainContentSelected = true;
    		this._altContentSelected = false;
    		this._historyContentSelected = false;    		

    	}       
    },
	
    isMainContentSelected: function() {
        return this._mainContentSelected;
    },	

    _selectAltContent: function() {
    	if (this.useDualContents || this.historyActivated || this.approvalActivated) {
    		this.defaultMainContent = false;
    		
    		if (this.containerNode ) {
    			this.mainContentSelector.set('label', this.mainContentDeselectedLabel);
    			this.containerNode.style.display = "none";
    		}            
    			this.altContainerNode.style.display = "block";
    		if(this.historyActivated && this.historyContainerNode){
    			this.historyContentSelector.set('label', this.historyContentDeselectedLabel);
    			this.historyContainerNode.style.display = "none";
    		}

    		//set which content is selected
    		this._altContentSelected = true;
    		this._mainContentSelected = false;    
    		this._historyContentSelected = false;    		

    	}
    },
	
	isAltContentSelected: function() {
        return this._altContentSelected;
    },
    
	_selectHistoryContent: function()
	{
		if (this.useDualContents ||  this.approvalActivated) {
			this.defaultMainContent = false;

			if (this.containerNode) {
				this.mainContentSelector.set('label', this.mainContentDeselectedLabel);
				this.containerNode.style.display = "none";
			}
			if(this.altContentMarkerEnabled && this.altContainerNode){                
				this.altContentSelector.set('label', this.altContentDeselectedLabel);
				this.altContainerNode.style.display = "none";
			}                

			if(this.historyContainerNode){
				this.historyContentSelector.set('label', this.historyContentSelectedLabel);
				this.historyContainerNode.style.display = "block";
			}

			
			//set which content is selected
			this._altContentSelected = false;
			this._mainContentSelected = false;      
			this._historyContentSelected = true;				
		}
	},
	
	 isHistoryContentSelected: function() {
        return this._historyContentSelected;
    },
	    
    getInfoAttributesNode: function() {
        return this.attributesNode;
    },
    
    getExtraAttributesNode: function() {
        return this.extraAttributesNode;
    },
    
    // Show or hide the info attributes area.
    showInfoAttributesNode: function(show) {
        var node = (this.splitAttributeArea) ? this.splitAttributeNode : this.attributesNode;
        _dojo.toggleClass(node, 'hidden', !show);
        _dojo.toggleClass(this.extraAttributesNode, 'hidden', !show);
    },
    getExtInfoAttributesNode: function() {
        return this.attributesNodeExt;
    },
    
    addInfoLineBreak: function(/* optional */ position) {
        var br = document.createElement("span");
        br.innerHTML = "<br>";
        var childrenAttributes = this.attributesNode.childNodes;
        if (!position || (childrenAttributes.length < 1) || (position >= childrenAttributes.length)) {
            // append to the end if position is not specified or no children attribute
            // or position is out of range
            this.attributesNode.appendChild(br);
        } else {
            // insert before the child attribute of specified position
            this.attributesNode.insertBefore(br, childrenAttributes[position]);
        }
    },
    
    _createInfoAttributeNode: function(labelDomNode, /*domNode or Widget*/valueDomNode, /*optional*/existingRow) {
        var tabNode = null;
        var valueNode = valueDomNode;
        var addTabPos = true;

        // this is a widget, get its domNode to display
        // if the widget provides a tab input node, get it
        if(valueDomNode.domNode) {
            valueNode = valueDomNode.domNode;
            if(valueDomNode.getTabInputNode) {
                tabNode = valueDomNode.getTabInputNode();
            }
        }        
        if( valueNode.nodeType == 3 )
        {
            //Cannot set the className on text nodes ( type 3 ) in IE
            valueNode = document.createElement( "span" );
            valueNode.appendChild( valueDomNode );
            addTabPos = false;
        }
        // Make all info attributes tab after the title editor
        if (addTabPos) {
            if (!tabNode) {
                tabNode = valueNode;
            }
            tabNode.tabIndex = 0;
        }
        _dojo.addClass(labelDomNode, "attributes-entry-label");
        _dojo.addClass(valueNode, "attributes-entry-value");

        // add the label & value dom nodes to the group
        // management.
        if(this.splitAttributeArea) {
            var content = {
                labelContent: labelDomNode, 
                content: valueNode
            };
            // append to an existing row, just added to the row
            if(existingRow) {
                this.attributesGroup.appendContentToField(existingRow, content);
                return existingRow;
            }
            var container = this.attributesGroup.addContent(content);
            this.attributesGroup.alignLabels();
            return container;
        }
        
        var attributeContainer = document.createElement("span");
        attributeContainer.className = "attributes-entry";
       
        attributeContainer.appendChild(labelDomNode);
        attributeContainer.appendChild(valueNode);
        return attributeContainer;
    },
    
    // Clears the InfoAttributes container. Remove all contents from the node.
    //
    clearInfoAttributes: function() {
        if(this.attributesNode) {
            Utility.destroyWidgetsFromNode(this.attributesNode);
            _dojo.empty(this.attributesNode);
        }        
        if (this.splitAttributeArea) {
            this.attributesGroup = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormFieldGroup();
			if(this.attributesGroup != null && this.splitAttributesNodeLeft !=null){
            	_dojo.place(this.attributesGroup.domNode, this.splitAttributesNodeLeft);
			}
        }
        this.clearExtraInfoAttributes();
    },

    // Clears the Extra InfoAttributes container. Remove all contents from the node.
    clearExtraInfoAttributes: function() {
		if(this.extraAttributesNode){
        	Utility.destroyWidgetsFromNode(this.extraAttributesNode);
       		_dojo.empty(this.extraAttributesNode);
		}
    },
    
    addInfoAttribute: function(labelDomNode, /*domNode or Widget*/valueDomNode, /* optional */ position) {
        var attributeContainer = this._createInfoAttributeNode(labelDomNode, valueDomNode);
        
        if (this.splitAttributeArea) {
            return attributeContainer;
        }        
        var childrenAttributes = this.attributesNode.childNodes;
        if (!position || (childrenAttributes.length < 1) || (position >= childrenAttributes.length)) {
            // append to the end if position is not specified or no children attribute
            // or position is out of range
            this.attributesNode.appendChild(attributeContainer);
        } else {
            // insert before the child attribute of specified position
            this.attributesNode.insertBefore(attributeContainer, childrenAttributes[position]);
        }
		return attributeContainer;
    },
    
    // Appends to an existing info attribute. This is only applicable in split attribute area
    // mode where info attributes are layout in a single column. In other mode, the client can just
    // use addInfoAttribute to append on info attributes.
    //
    appendToExistingInfoAttribute: function(attrEntry, labelDomNode, /*domNode or Widget*/valueDomNode) {
        if(this.splitAttributeArea) {
            return this._createInfoAttributeNode(labelDomNode, valueDomNode, attrEntry);
        }
        return null;
    },
    
    setExtInfoAttribute: function(labelDomNode, /*domNode or Widget*/valueDomNode) {
        var attributeContainer = this._createInfoAttributeNode(labelDomNode, valueDomNode);
        dojox.xml.parser.removeChildren(this.attributesNodeExt);
        this.attributesNodeExt.appendChild(attributeContainer);
        return attributeContainer;
    },
    
    getSummaryNode: function() {
        return this.summaryNode;
    },

    getleftActionsContainer: function() {
        return this.actionsAreaLeft;
    },

    getRightActionsContainer: function() {
        return this.actionsAreaRight;
    },
	
	getleftActionsBottomContainer: function() {
        return this.actionsBottomAreaLeft;
    },

    getRightActionsBottomContainer: function() {
        return this.actionsBottomAreaRight;
    },
    
    onTitleEdited: function(value) {
        // called when the title (editable) has been modified by the user.    
    },

    onSummaryEdited: function(value) {
        // called when the summary (editable) has been modified by the user.    
    },
     // Summary:
    //    set the minimum Height required by the Header this is important since editable
    //    Title chenged the size of the Header and this make the click event fail when click a different action
    //
    setHeight: function(height) {
        if(height){
            this.minHeight = height;
        }
        if (this.minHeight == ViewHeaderLayout.LARGE ) {
            _dojo.addClass(this.viewHeaderBackNode, "min-height-large");
        } else if (this.minHeight == ViewHeaderLayout.MEDIUM) {
            _dojo.addClass(this.viewHeaderBackNode, "min-height-medium");
        } else if (this.minHeight == ViewHeaderLayout.SMALL) {
            _dojo.addClass(this.viewHeaderBackNode, "min-height-small");
        }
   },
   
   startValidating: function() {
	   this.validationStarted = true;
	   this._updateTitleRequiredError();
	   this._updateSummaryRequiredError();
   }
    
});

/***
 * layout used in the all artifacts listing view.  The status box is relocated to the left
 * of the actions area.
 */
dojo.declare("com.ibm.asq.common.web.ui.internal.view.layout.ListingViewHeaderLayout",
com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout,
{
    isCompact: true,
    noline: true,
    isMinMarginLeft: true,
    containerWrapper: false,  // if true, create a wrapper DIV around the containerNode
    isEditor: false,
    
    // IE needs block display style to place the vertical position properly. There is no
    // floating actions for the title to run into in the listing view anyway
    titleDisplayStyle: _dojo.isIE ? "inline-block" : "inline",

    _trimStatusTextMax: 65,

    postCreate: function() {
        this.inherited(arguments);
        var marginLeftIE = "3px";
        if (this.containerWrapper) {
            // create a wrapper DIV around the containerNode
            this.containerWrapperNode = _dojo.create("div", {
                style: "padding:1px; margin-left:2px;"
            }); 
            _dojo.place(this.containerWrapperNode, this.containerNode, 'replace');
            _dojo.place(this.containerNode, this.containerWrapperNode);
        } else if (_dojo.isIE) {
            _dojo.style(this.containerNode, "marginLeft", "3px");
            _dojo.style(this.viewHeaderLeft, "marginTop", "5px");
            marginLeftIE = "4px";
        }
        // pixels tweaking
        _dojo.style(this.viewHeaderLeft, "marginLeft", _dojo.isIE ? marginLeftIE : "3px");
        _dojo.style(this.breadcrumbContainerNode, "marginLeft", "3px");
        
        // relocate the status box to the top and bottom next to the action buttons
        var updatedMessageBoxStyle = { "clear":"none","margin":"0px -5px","float":"right","width":"auto" };
        _dojo.place(this.wholestatusContainer, this.actionsAreaLeft, 'after');
        _dojo.style(this.wholestatusContainer, updatedMessageBoxStyle);
        _dojo.place(this.wholestatusBottomContainer, this.actionsBottomAreaLeft, 'after');
        _dojo.style(this.wholestatusBottomContainer, updatedMessageBoxStyle);
        
        // If the top status line is expanded to show details, make the line not
        // float. This would allow it to assume normal flow.
        this.statusExpanded = false;
        this.statusLineNode.onToggle = _dojo.hitch(this, function() {
            var expanded = (this.statusLineNode.toggleState == "visible");
            var expandTo = (this.leftOnDetails) ? "85%" : "100%";
            
            this.statusExpanded = expanded;
            _dojo.style(this.wholestatusContainer, {
                'float': expanded ? "" : "right",
                // don't expand all the way. This would cover the action area and
                // made the right view actions not clickable.
                'width': expanded ? expandTo : "auto"
            });
        });
    },

    // @override
    setStatus: function(status, type, /*optional*/details, /*optional*/errorObj) {
        this.inherited(arguments);
        
        // if our status line was expanded, need to reset the style
        // if we don't have any details/errors in the new message.
        if(this.statusExpanded && (!details || !errorObj)) {
            this._resetStatusStyle();
        }
    },
    
    // @override
    clearStatus: function() {
        this.inherited(arguments);
        this._resetStatusStyle();
    },
    
    _resetStatusStyle: function() {
        _dojo.style(this.wholestatusContainer, {
            'float': "right",
            'width':"auto"
        });
        this.statusExpanded = false;
    },
    
    setBreadcrumb: function(crumbs) {
        this.inherited(arguments);
        var hasContent = crumbs && _dojo.isArray(crumbs) && (crumbs.length > 0);
        if (_dojo.isIE) {
            // Seems like IE clips the margin of the inner breadcrumbs widget if it is not empty,
            // but we don't want the extra margin if it is empty.  What a pain!
            var crumbsOffset = hasContent ? {"marginTop":"-4px","marginBottom":"10px"} : {"marginTop":"0px","marginBottom":"0px"};
            _dojo.style(this.breadcrumbContainerNode, crumbsOffset);
        }
        _dojo.style(this.viewHeaderBackNode, "marginTop", hasContent ? "3px" : "");
    }
});
   

//	Create some definitions that will be used in other files
//	Note that the ViewHeaderLayout variable is used locally to this file as a shortcut.
	var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	ViewHeaderLayout.ERROR 		= 1;
	ViewHeaderLayout.SUCCESS 	= 2;
	ViewHeaderLayout.INFO 	= 3;
	ViewHeaderLayout.WARNING	= 4;
	ViewHeaderLayout.LARGE        = 1;
    ViewHeaderLayout.MEDIUM    = 2;
    ViewHeaderLayout.SMALL   = 3;
})();